#! -*- coding: UTF-8 -*-

from zope.interface import Interface, alsoProvides
from zope.app.content.interfaces import IContentType

class IHotSite(Interface):
    ''' A section of your Plone Site that act as a Hot Site '''

alsoProvides(IHotSite, IContentType)

class INoHotSite(Interface):
    ''' A marker interface that allows subtyping reset '''

alsoProvides(INoHotSite, IContentType)



from zope.schema import Int
from zope.schema import Bool
from zope.schema import ASCII
from zope.schema import Tuple
from zope.schema import TextLine
from zope.formlib import form
from zope.component import adapts
from zope.interface import Interface
from zope.interface import implements
from zope.app.form.browser.textwidgets import ASCIIWidget
from zope.app.form.browser.boolwidgets import CheckBoxWidget

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFDefault.formlib.schema import ProxyFieldProperty
from Products.CMFDefault.formlib.schema import SchemaAdapterBase

from sc.base.hotsites import MessageFactory as _

from plone.fieldsets import FormFieldsets
from plone.app.controlpanel.form import ControlPanelForm


class IHotSitesConfSchema(Interface):

    allowedContentTypes = Tuple(
        title=_(u'Allowed Content Types'),
        description=_(u'allowed_content_types',
            default=u"Allowed Content Types on Hot Sites folder"
        ),
        value_type=TextLine(),
        default=(),
        required=False,
    )

    skin_name = ASCII(
        title=_(u'label_skin_name',
            default=u'Skin Name',
        ),
        description=_(u'help_skin_name',
            default=u"Skin name to use on Hot Sites folder "
        ),
        default=None,
        required=True,
    )

    default_view_id = ASCII(
        title=_(u'label_default_view_id',
            default=u'Default view to the hot site',
        ),
        description=_(u'help_default_view_id',
            default=u"Id of object to be used as default view on the hot site"
        ),
        default=None,
        required=False,
    )

    default_view_path = ASCII(
        title=_(u'label_default_view_path',
            default=u'Path of object',
        ),
        description=_(u'help_default_view_path',
            default=u"Path of object to be copied as default view of hot site",
        ),
        default=None,
        required=False,
    )

    workflow_id = ASCII(
        title=_(u'label_workflow_id',
            default=u'Workflow Id to use on hot site',
        ),
        description=_(u'help_workflow_id',
            default=u"Id of the workflow to be used in the hot site folder",
        ),
        default=None,
        required=False,
    )

    use_accessibility = Bool(
        title=_(u'label_use_accessibility',
            default=u'Use a alterantive document as accessibility content?',
        ),
        description=_(u'help_use_accessibility',
            default=u"If checked, will be created a document to be used as accessibility content.",
        ),
        default=False,
        required=False,
    )

class HotSitesControlPanelAdapter(SchemaAdapterBase):

    adapts(IPloneSiteRoot)
    implements(IHotSitesConfSchema)

    def __init__(self, context):
        super(HotSitesControlPanelAdapter, self).__init__(context)
        portal_properties = getToolByName(context, 'portal_properties')
        self.context = portal_properties.hotsites_conf

    allowedContentTypes = ProxyFieldProperty(IHotSitesConfSchema['allowedContentTypes'])
    skin_name = ProxyFieldProperty(IHotSitesConfSchema['skin_name'])
    default_view_id = ProxyFieldProperty(IHotSitesConfSchema['default_view_id'])
    default_view_path = ProxyFieldProperty(IHotSitesConfSchema['default_view_path'])
    workflow_id = ProxyFieldProperty(IHotSitesConfSchema['workflow_id'])
    use_accessibility = ProxyFieldProperty(IHotSitesConfSchema['use_accessibility'])
    #viewlest_to_hide = ProxyFieldProperty(IHotSitesConfSchema['viewlest_to_hide'])

hotsites_set = FormFieldsets(IHotSitesConfSchema)
hotsites_set.id = 'hotsites'
hotsites_set.label = _(u'Hot Sites')
class HotSitesControlPanel(ControlPanelForm):

    form_fields = FormFieldsets(hotsites_set)
    #form_fields['allowedContentTypes'].custom_widget = ASCIIWidget
    form_fields['skin_name'].custom_widget = ASCIIWidget
    form_fields['default_view_id'].custom_widget = ASCIIWidget
    form_fields['default_view_path'].custom_widget = ASCIIWidget
    form_fields['workflow_id'].custom_widget = ASCIIWidget
    form_fields['use_accessibility'].custom_widget = CheckBoxWidget
    #form_fields['viewlest_to_hide'].custom_widget = ASCIIWidget
    label = _('Hot Site Settings')
    description = _('Settings to Hot Site.')
    form_name = _('Hot Site settings')



#! -*- coding: UTF-8 -*-

from zope.component import getMultiAdapter, getUtility
from sc.base.hotsites.interfaces import IHotSite, INoHotSite
from sc.base.hotsites import MessageFactory as _
from Products.CMFPlacefulWorkflow.PlacefulWorkflowTool import WorkflowPolicyConfig_id
from Products.CMFCore.utils import getToolByName
from zope.annotation.interfaces import IAnnotations
from plone.portlets.interfaces import IPortletManager, ILocalPortletAssignmentManager
from plone.portlets.constants import CONTENT_TYPE_CATEGORY, CONTEXT_CATEGORY, GROUP_CATEGORY
from Acquisition import aq_inner, aq_parent
from Products.statusmessages.interfaces import IStatusMessage


K_PREFIX = 'sc.hotsite'
def newHotSite(obj):
    folder = obj.object
    if IHotSite.providedBy(folder):

        props = getToolByName(folder, 'portal_properties')
        confs = props.hotsites_conf
        allowedCTs = confs.allowedContentTypes
        skin_name = confs.skin_name
        default_view_id = confs.default_view_id
        default_view_path = confs.default_view_path
        workflow_id = confs.workflow_id
        use_accessibility = confs.use_accessibility

        # block all portlets in the folder
        # saves a tuple with original block state
        value = getBlockedPortlets(folder, 'plone.rightcolumn')
        save_value(folder, 'plone.rightcolumn', value)
        value = getBlockedPortlets(folder, 'plone.leftcolumn')
        save_value(folder, 'plone.leftcolumn', value)

        portlet_blacklist = folder.restrictedTraverse('set-portlet-blacklist-status') 
        portlet_blacklist(manager='plone.rightcolumn',  group_status=1, content_type_status=1, context_status=1)
        portlet_blacklist(manager='plone.leftcolumn',  group_status=1, content_type_status=1, context_status=1)


        # Change the locally allowed types
        save_value(folder, 'constrainTypesMode', folder.getConstrainTypesMode())
        save_value(folder, 'locallyAllowedTypes', folder.getLocallyAllowedTypes())
        save_value(folder, 'immediatelyAddableTypes', folder.getImmediatelyAddableTypes())
        folder.setConstrainTypesMode(1)
        folder.setLocallyAllowedTypes(allowedCTs)
        folder.setImmediatelyAddableTypes(allowedCTs)

        # Set a access_role to change the skin
        script_body = '''
context.changeSkin('%s', context.REQUEST)
return 1
        ''' % skin_name
        if not 'setSkin' in folder.objectIds():
            folder.manage_addProduct['PythonScripts'].manage_addPythonScript('setSkin')
        folder['setSkin'].write(script_body)
        folder.manage_addProduct['SiteAccess'].manage_addAccessRule('setSkin')

        change_default_view = False
        if not default_view_id in folder.objectIds():
            if default_view_path:
                obj_tmpl = folder.unrestrictedTraverse(default_view_path, None)
                if obj_tmpl:
                    container = aq_parent(aq_inner(obj_tmpl))
                    obj = container.manage_copyObjects([obj_tmpl.getId()])
                    results = folder.manage_pasteObjects(obj)
                    nw_obj = [r for r in results if r['id'] == obj_tmpl.getId()]
                    nw_obj = nw_obj[0]
                    nw_obj_id = nw_obj['new_id']

                    folder.manage_renameObjects([nw_obj_id], [default_view_id])

                    nw_obj = folder[default_view_id]

                    wt = getToolByName(folder, 'portal_workflow')
                    if 'publish' in [w['id'] for w in  wt.getTransitionsFor(nw_obj)] :
                        wt.doActionFor(nw_obj, 'publish')
                    nw_obj.reindexObject()
                    change_default_view = True
                else:
                    msg = _(u'Default view template ${path} not found.', 
                            mapping={'path': default_view_path})
                    IStatusMessage(folder.request).addStatusMessage(msg, type='info')
        else:
            change_default_view = True

        # Creates a document to be the accessibility content
        if use_accessibility and not 'accessibility' in folder.objectIds():
            folder.invokeFactory('Document', id='accessibility')
            
        # change the default view
        save_value(folder, 'layout', folder.getLayout())
        if change_default_view:
            folder.setLayout(default_view_id)

        # Set the local workflow to workflow_id from confs
        if not WorkflowPolicyConfig_id in folder.objectIds():
            folder.manage_addProduct['CMFPlacefulWorkflow'].manage_addWorkflowPolicyConfig()
            config = getattr(folder, WorkflowPolicyConfig_id)
        else:
            config = getToolByName(folder, 'portal_placeful_workflow').getWorkflowPolicyConfig(folder)
        config.setPolicyBelow(workflow_id)
        getToolByName(folder, 'portal_workflow').updateRoleMappings()

    elif INoHotSite.providedBy(folder):

        portlet_blacklist = folder.restrictedTraverse('set-portlet-blacklist-status') 

        c_content, c_context, c_group = get_value(folder, 'plone.rightcolumn')
        portlet_blacklist(manager='plone.rightcolumn', group_status=c_group, content_type_status=c_content, context_status=c_content)
        c_content, c_context, c_group = get_value(folder, 'plone.leftcolumn')
        portlet_blacklist(manager='plone.leftcolumn',  group_status=c_group, content_type_status=c_content, context_status=c_content)


        visao_padrao = get_value(folder, 'layout')
        if visao_padrao is not None:
            folder.setLayout(visao_padrao)

        # get back the values to allowed types
        constrainTypesMode = get_value(folder, 'constrainTypesMode')
        if constrainTypesMode is not None:
            folder.setConstrainTypesMode(constrainTypesMode)
        locallyAllowedTypes = get_value(folder, 'locallyAllowedTypes')
        if locallyAllowedTypes is not None:
            folder.setLocallyAllowedTypes(locallyAllowedTypes)
        immediatelyAddableTypes = get_value(folder, 'immediatelyAddableTypes')
        if immediatelyAddableTypes is not None:
            folder.setImmediatelyAddableTypes(immediatelyAddableTypes)

        # removes the access rule, so this get back the default skin
        folder.manage_addProduct['SiteAccess'].manage_addAccessRule(None)
        if 'setSkin' in folder.objectIds():
            pass
            #folder.manager_delObject(

        # Workflow local
        # get back the default workflow
        if WorkflowPolicyConfig_id in folder.objectIds():
            config = getToolByName(folder, 'portal_placeful_workflow').getWorkflowPolicyConfig(folder)
            config.setPolicyBelow('default_policy')  #default_policy
            getToolByName(folder, 'portal_workflow').updateRoleMappings()


def save_value(obj, key, value):
    ''' save the value in the obj, with the key using Annotation,
    using a prefix to the key
    '''
    ann_obj = IAnnotations(obj)
    ann_obj['%s.%s' % (K_PREFIX, key)] = value


def get_value(obj, key):
    ''' Returns the value from obj, using the Annoation with a key prefix
    '''
    ann_obj = IAnnotations(obj)
    return ann_obj.get('%s.%s' % (K_PREFIX, key), None)

def getBlockedPortlets(context, manager_name):
    ''' Returns the blocked configuration of portlets in the context
    returns a tuple (CONTENT_TYPE_CATEGORY, CONTEXT_CATEGORY, GROUP_CATEGORY) '''
    
    manager = getUtility(IPortletManager, manager_name) 

    assignable = getMultiAdapter((context, manager), ILocalPortletAssignmentManager)

    c_content = assignable.getBlacklistStatus(CONTENT_TYPE_CATEGORY)
    c_context = assignable.getBlacklistStatus(CONTEXT_CATEGORY)
    c_group = assignable.getBlacklistStatus(GROUP_CATEGORY)

    return (c_content, c_context, c_group)





#! -*- coding: UTF-8 -*-

from p4a.subtyper.interfaces import IPortalTypedFolderishDescriptor
from zope.interface import implements

from zope.interface import Interface, alsoProvides
from zope.app.content.interfaces import IContentType

from sc.base.hotsites.interfaces import IHotSite, INoHotSite
from sc.base.hotsites import MessageFactory as _

class HotSite(object):
    implements(IPortalTypedFolderishDescriptor)

    title = _(u'Hot Site')
    description = _(u'Website section that acts as a Hot Site')
    type_interface = IHotSite
    for_portal_type = 'Folder'


class NoHotSite(object):
    implements(IPortalTypedFolderishDescriptor)

    title = _(u'Normal Folder')
    description = _(u'Just a plain Folder that does NOT acts as a Hot Site')
    type_interface = INoHotSite
    for_portal_type = 'Folder'



# -*- coding: utf-8 -*-

from zope.i18nmessageid import MessageFactory as BaseMessageFactory
MessageFactory = BaseMessageFactory('sc.base.hotsites')

def initialize(context):
    """Initializer called when used as a Zope 2 product."""



# See http://peak.telecommunity.com/DevCenter/setuptools#namespace-packages
try:
    __import__('pkg_resources').declare_namespace(__name__)
except ImportError:
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)

try:
    from zopeskel.localcommands.plone import *
    from zopeskel.localcommands.archetype import *

    ArchetypeSubTemplate.parent_templates.append('sc_plone_app')
    PloneSubTemplate.parent_templates.append('sc_plone_app')    
except ImportError:
    pass



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os

version = open(os.path.join("sc", "base", "hotsites", "version.txt")).read().strip()

setup(name='sc.base.hotsites',
      version=version,
      description="",
      long_description=open(os.path.join("sc", "base", "hotsites", "README.txt")).read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Simples Consultoria',
      author_email='products@simplesconsultoria.com.br',
      url='http://www.simplesconsultoria.com.br/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['sc', 'sc.base'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'p4a.subtyper',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-

      [distutils.setup_keywords]
      paster_plugins = setuptools.dist:assert_string_list

      [egg_info.writers]
      paster_plugins.txt = setuptools.command.egg_info:write_arg
      """,
      paster_plugins = ["ZopeSkel"],
      )



import unittest
import doctest

from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import PloneSite
from Products.PloneTestCase.layer import onsetup

import sc.base.hotsites

@onsetup
def setup_product():
    fiveconfigure.debug_mode = True
    zcml.load_config('configure.zcml',
                     sc.base.hotsites)
    fiveconfigure.debug_mode = False
    ztc.installPackage('sc.base.hotsites')

setup_product()

ptc.setupPloneSite(extension_profiles=['sc.base.hotsites:default'])

class TestCase(ptc.PloneTestCase):
    class layer(PloneSite):

        @classmethod
        def tearDown(cls):
            pass



def test_suite():
    return unittest.TestSuite([

        # Unit tests
        #doctestunit.DocFileSuite(
        #    'README.txt', package='sc.base.hotsites',
        #    setUp=testing.setUp, tearDown=testing.tearDown),

        #doctestunit.DocTestSuite(
        #    module='sc.base.hotsites.mymodule',
        #    setUp=testing.setUp, tearDown=testing.tearDown),


        # Integration tests that use PloneTestCase
        ztc.ZopeDocFileSuite(
            'README.txt', package='sc.base.hotsites.docs',
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE | 
                         doctest.NORMALIZE_WHITESPACE | 
                         doctest.ELLIPSIS,
            test_class=TestCase),

        ztc.FunctionalDocFileSuite(
            'browser.txt', 
            package='sc.base.hotsites.docs',
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE | 
                        doctest.NORMALIZE_WHITESPACE | 
                        doctest.ELLIPSIS,
            test_class=TestCase),

        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
